*  LET status=LEVEL2%(#channel)
*  by Dilwyn Jones, 2018. Freeware. v1.01
*  Based on a function in Norman Dunbar's DJToolkit

*  Link to QLiberator compiled programs with
*  $$asmb=dddn_level2_cde,0,12
*  Link to Turbo compiled programs with
*  %%dddn_level2_cde,0,12

*  Link in the new keyword
Start     lea       table,a1
          move.w    $0110,a2
          jsr       (a2)
          rts

*  No procedures
table     dc.w      0               ;no procedures
          dc.w      0               ;end of procedures
*  1 function
          dc.w      1               ;1 function
          dc.w      level2-*
          dc.b      7,'LEVEL2%'     ;'%' avoids name clash with DJTK
          dc.w      0               ;end of functions

* count parameters (hope for just 1)
level2    move.l    a5,d0           ;pointer to last parameter
          sub.l     a3,d0           ;pointer to first parameter
          divu      #8,d0           ;8 bytes per entry, how many?
          subq.w    #1,d0           ;was there 1 parameter?
          beq.s     onepar          ;yes, 1 parameter

errbp     moveq     #-15,d0         ;oops, err_bp (-15) bad parameter
return    rts                       ;return with error

* test for # in front of channel number
onepar    btst      #7,1(a6,a3.l)   ;test for hash flag for channel number
          beq.s     errbp           ;not there, quit and complain

* try to fetch the channel number parameter
          move.w    $118,a0         ;ca.gtlin fetch long integer parameter
          jsr       (a0)            ;fetch channel number parameter
          bne.s     errbp           ;oops, bad parameter
          move.l    0(a6,a1.l),d0   ;get channel number from stack
          adda.l    #2,a1           ;tidy stack, leaving room for word
          move.l    a1,$58(a6)      ;store RI stack pointer in bv.rip

* check that the channel is actually open
          tst.l     d0              ;test channel id in d0
          bmi.s     chnotopen       ;err.bp value returned if negative
          mulu      #$28,d0         ;channel ID * ch.lench
          add.l     $30(a6),d0      ;BV.CHBAS, base of channel table
          cmp.l     $34(a6),d0      ;BV.CHP, top of channel table
          bge.s     chnotopen       ;oops, channel not open
          movea.l   0(a6,d0.l),a0   ;channel ID
          tst.l     0(a6,d0.l)      ;is it 0 or positive?
          bmi.s     chnotopen       ;oops, channel not open

* channel ID in d0, save temporarily
          move.l    a0,d7           ;save channel ID in d7

* need a 64 byte buffer for iof.xinf
          moveq     #$18,d0         ;mt.alchp - allocate space in heap
          moveq     #64,d1          ;64 byte buffer needed
          moveq     #-1,d2          ;owned by myself
          trap      #1              ;call mt.alchp to make 64 byte buffer
          tst.w     d0              ;any error? e.g. out of memory
          bne.s     retvalue        ;oops, return d0 as function value

* buffer address returned in a0

* iof.xinf trap #3, d0=$4F  Get extended information
* Call parameters       Return parameters
* D1   0                 D1   preserved
* D2                     D2   preserved
* D3.w timeout           D3   preservd
* A0   channel ID        A0   preserved
* A1   pointer to info   A1   preserved
*      buffer 64 bytes
* call with d2.b=0 to check if level 2 facilities exist.
* If no info block returned, no level 2 facilities.
          moveq     #$4f,d0         ;use iof_xinf extended trap
          moveq     #0,d1           ;must be 0
          moveq     #0,d2           ;interrogate mode
          moveq     #-1,d3          ;infinite timeout
          movea.l   a0,a1           ;buffer address into A1 for trap
          move.l    d7,a0           ;recover channel ID from d7
          trap      #3              ;call iof.xinf
          move.w    d0,d4           ;store error code from iof.xinf

* we can get rid of the 64 byte buffer now
          moveq     #$19,d0         ;mt_rechp trap
          movea.l   a1,a0           ;buffer address
          trap      #1              ;release buffer, no errors

          move.w    d4,d0           ;restore previous error from iof.xinf
          beq.s     l2present       ;Level 2 drivers present

notl2     move.w    #0,d0           ;Level 2 drivers not found, return 0
          bra.s     retvalue        ;back to superbasic
l2present move.w    #1,d0           ;level 2 drivers present, return 1
          bra.s     retvalue        ;back to superbasic
chnotopen move.w    #-6,d0          ;error code for channel not open

retvalue  move.l    $58(a6),a1      ;recover RI stack pointer from bv.rip
          move.w    d0,0(a6,a1.l)   ;put return value on RI stack
          moveq     #3,d4           ;return an integer
          moveq     #0,d0           ;no error
          rts
